use std::{fs, path::PathBuf, process, env};

fn find_target_dir(path: PathBuf) -> Result<bool, std::io::Error> {
    let mut found_target_dir: bool = false;
    let mut found_cargo_toml: bool = false;
    for entry in fs::read_dir(path)? {
        let entry = entry?;
        let p = entry.path();
        if p.is_dir() && p.ends_with("target") {
            found_target_dir = true;
        } else if p.is_file() && p.file_name().unwrap() == "Cargo.toml" {
            found_cargo_toml = true;
        }
    }
    Ok(found_target_dir && found_cargo_toml)
}

fn execute_clean(path: PathBuf) -> Result<(), std::io::Error> {
    let _cmd = process::Command::new("cargo")
        .current_dir(path)
        .arg("clean")
        .status()?;
    Ok(())
}

fn clean_in_dir(path: PathBuf) -> Result<u64, std::io::Error> {
    let mut cleaned: u64 = 0;
    for entry in fs::read_dir(path)? {
        let entry = entry?;
        let p = entry.path();
        if p.is_dir() {
            let needed = find_target_dir(p.clone())?;
            if needed {
                println!(" => {}", p.display());
                execute_clean(p)?;
                cleaned += 1;
            } else {
                // Maybe nested?
                let r = clean_in_dir(p)?;
                cleaned += r;
            }
        }
    }
    Ok(cleaned)
}

fn main() -> Result<(), std::io::Error> {
    println!("Sweeping...");
    let cwd = env::current_dir()?;
    let cleaned = clean_in_dir(cwd)?;
    if cleaned == 1 {
        println!("Cleaned 1 crate");
    } else {
        println!("Cleaned {} crates", cleaned);
    }
    Ok(())
}
